@@ -231,6 +231,8 @@ type Server struct {
 	LogHeaders bool
 	// Disable transcoding, and the resource elements implied in the CDS.
 	NoTranscode bool
+	// Force transcoding to certain format of the 'transcodes' map
+	ForceTranscodeTo string
 	// Disable media probing with ffprobe
 	NoProbe bool
 	Icons   []Icon
@@ -243,6 +245,8 @@ type Server struct {
 	IgnoreHidden bool
 	// Ingnore unreadable files and directories
 	IgnoreUnreadable bool
+	// White list of clients
+	AllowedIpNets []*net.IPNet
 }
 
 // UPnP SOAP service.
@@ -524,6 +528,18 @@ func (me *Server) soapActionResponse(sa upnp.SoapAction, actionRequestXML []byte
 
 // Handle a service control HTTP request.
 func (me *Server) serviceControlHandler(w http.ResponseWriter, r *http.Request) {
+	found := false
+	clientIp, _, _ := net.SplitHostPort(r.RemoteAddr)
+	for _, ipnet := range me.AllowedIpNets {
+		if ipnet.Contains(net.ParseIP(clientIp)) {
+			found = true
+		}
+	}
+	if !found {
+		log.Printf("not allowed client %s, %+v", clientIp, me.AllowedIpNets)
+		http.Error(w, "forbidden", http.StatusForbidden)
+		return
+	}
 	soapActionString := r.Header.Get("SOAPACTION")
 	soapAction, err := upnp.ParseActionHTTPHeader(soapActionString)
 	if err != nil {
@@ -714,7 +730,12 @@ func (server *Server) initMux(mux *http.ServeMux) {
 			http.Error(w, "no such object", http.StatusNotFound)
 			return
 		}
-		k := r.URL.Query().Get("transcode")
+		var k string
+		if server.ForceTranscodeTo != "" {
+			k = server.ForceTranscodeTo
+		} else {
+			k = r.URL.Query().Get("transcode")
+		}
 		if k == "" {
 			mimeType, err := MimeTypeByPath(filePath)
 			if err != nil {
@@ -766,7 +787,7 @@ func (s *Server) initServices() (err error) {
 	return
 }
 
-func (srv *Server) Serve() (err error) {
+func (srv *Server) Init() (err error) {
 	if err = srv.initServices(); err != nil {
 		return
 	}
@@ -836,6 +857,20 @@ func (srv *Server) Serve() (err error) {
 	log.Println("HTTP srv on", srv.HTTPConn.Addr())
 	srv.initMux(srv.httpServeMux)
 	srv.ssdpStopped = make(chan struct{})
+	return nil
+}
+
+// Deprecated: Use Init and then Run. There's a race calling Close on a Server that's had Serve
+// called on it.
+func (srv *Server) Serve() (err error) {
+	err = srv.Init()
+	if err != nil {
+		return
+	}
+	return srv.Run()
+}
+
+func (srv *Server) Run() (err error) {
 	go func() {
 		srv.doSSDP()
 		close(srv.ssdpStopped)
